#ifndef _MOTION_MANAGER_H
#define _MOTION_MANAGER_H

#ifdef __cplusplus
extern "C" {
#endif

#include "stm32f4xx.h"

typedef enum {
  R_SHOULDER_PITCH     = 1,
  L_SHOULDER_PITCH     = 2,
  R_SHOULDER_ROLL      = 3,
  L_SHOULDER_ROLL      = 4,
  R_ELBOW              = 5,
  L_ELBOW              = 6,
  R_HIP_YAW            = 7,
  L_HIP_YAW            = 8,
  R_HIP_ROLL           = 9,
  L_HIP_ROLL           = 10,
  R_HIP_PITCH          = 11,
  L_HIP_PITCH          = 12,
  R_KNEE               = 13,
  L_KNEE               = 14,
  R_ANKLE_PITCH        = 15,
  L_ANKLE_PITCH        = 16,
  R_ANKLE_ROLL         = 17,
  L_ANKLE_ROLL         = 18} servo_id_t;


typedef enum {MM_NONE          = 0,
              MM_ACTION        = 1,
              MM_WALKING       = 2,
              MM_JOINTS        = 3} TModules;

typedef struct
{
  void (*process_fnct)(void);
  void (*init_fnct)(void);
  uint8_t name[16];
}TMotionModule;

#define MAX_MOTION_MODULES              8

// servo information structure
typedef struct{
  uint8_t id;
  uint16_t model;
  uint16_t encoder_resolution;
  uint8_t gear_ratio;
  uint16_t max_angle;
  uint16_t center_angle;
  uint16_t max_speed;
  int16_t current_angle;
  uint16_t current_value;
  TModules module;
  uint8_t enabled;
  int16_t cw_angle_limit;
  int16_t ccw_angle_limit;
  uint8_t dyn_version;
}TServoInfo;

#define MANAGER_MAX_NUM_SERVOS          31

// public variables
extern int64_t manager_current_angles[MANAGER_MAX_NUM_SERVOS];

// public functions
void manager_init(uint16_t period_us);
inline uint16_t manager_get_period(void);
inline uint16_t manager_get_period_us(void);
inline void manager_set_period(uint16_t period_us);
inline void manager_enable(void);
inline void manager_disable(void);
inline uint8_t manager_is_enabled(void);
void manager_enable_balance(void);
void manager_disable_balance(void);
inline uint8_t manager_get_num_servos(void);
void manager_set_module(uint8_t servo_id,TModules module);
inline TModules manager_get_module(uint8_t servo_id);
inline void manager_enable_servo(uint8_t servo_id);
inline void manager_disable_servo(uint8_t servo_id);
inline uint8_t manager_is_servo_enabled(uint8_t servo_id);
inline int16_t manager_get_cw_angle_limit(uint8_t servo_id);
inline int16_t manager_get_ccw_angle_limit(uint8_t servo_id);
// motion modules handling
//void manager_add_module(
// operation functions
uint8_t manager_in_range(unsigned short int address, unsigned short int length);
void manager_process_write_cmd(unsigned short int address,unsigned short int length,unsigned char *data);

#ifdef __cplusplus
}
#endif

#endif
