#include "adc_dma.h"
#include "ram.h"
 
#define ADC1_CH1                    ADC_CHANNEL_6
#define ADC1_CH2                    ADC_CHANNEL_8
#define ADC1_CH3                    ADC_CHANNEL_TEMPSENSOR
#define ADC1_CH4                    ADC_CHANNEL_VREFINT

#define ADC2_CH1                    ADC_CHANNEL_7
#define ADC2_CH2                    ADC_CHANNEL_9
#define ADC2_CH3                    ADC_CHANNEL_14
#define ADC2_CH4                    ADC_CHANNEL_15

#define ADC1_CH1_PIN                GPIO_PIN_6
#define ADC1_CH1_PORT               GPIOA
#define ADC1_CH1_ENABLE_PORT_CLK    __HAL_RCC_GPIOA_CLK_ENABLE()
#define ADC1_CH2_PIN                GPIO_PIN_0
#define ADC1_CH2_PORT               GPIOB
#define ADC1_CH2_ENABLE_PORT_CLK    __HAL_RCC_GPIOB_CLK_ENABLE()
#define ADC1_ENABLE_CLK             __ADC1_CLK_ENABLE()

#define ADC2_CH1_PIN                GPIO_PIN_7
#define ADC2_CH1_PORT               GPIOA
#define ADC2_CH1_ENABLE_PORT_CLK    __HAL_RCC_GPIOA_CLK_ENABLE()
#define ADC2_CH2_PIN                GPIO_PIN_1
#define ADC2_CH2_PORT               GPIOB
#define ADC2_CH2_ENABLE_PORT_CLK    __HAL_RCC_GPIOB_CLK_ENABLE()
#define ADC2_CH3_PIN                GPIO_PIN_4
#define ADC2_CH3_PORT               GPIOC
#define ADC2_CH3_ENABLE_PORT_CLK    __HAL_RCC_GPIOC_CLK_ENABLE()
#define ADC2_CH4_PIN                GPIO_PIN_5
#define ADC2_CH4_PORT               GPIOC
#define ADC2_CH4_ENABLE_PORT_CLK    __HAL_RCC_GPIOC_CLK_ENABLE()
#define ADC2_ENABLE_CLK             __ADC2_CLK_ENABLE()

#define ADC_DMA_CHANNEL             DMA_CHANNEL_0
#define ADC_DMA_STREAM              DMA2_Stream0
#define ADC_DMA_IRQn                DMA2_Stream0_IRQn
#define ADC_DMA_IRQHandler          DMA2_Stream0_IRQHandler
#define ADC_ENABLE_DMA_CLK          __HAL_RCC_DMA2_CLK_ENABLE()

#define ADC_TIMER                   TIM4
#define ENABLE_ADC_TIMER_CLK        __HAL_RCC_TIM4_CLK_ENABLE()
#define ADC_TIMER_IRQn              TIM4_IRQn
#define ADC_TIMER_IRQHandler        TIM4_IRQHandler

// temperature conversion functions
#define TEMP_V25                    0.76// ADC voltage at 25 degrees in V
#define TEMP_INV_SLOPE              400.0// ADC slope in mV/degree

// general voltage conversion
#define VOLTAGE_DELTA               0.000805664
#define VOLTAGE_GAIN                1.529411765

// private variables
ADC_HandleTypeDef hadc1;
ADC_HandleTypeDef hadc2;
DMA_HandleTypeDef hdma_adc1; 
TIM_HandleTypeDef ADC_TIM_Handle;

uint32_t adc_data[4];// temporal buffer to store ADC data before conversion
uint16_t adc_period_ms;

// private functions
uint16_t adc_convert_temperature(uint16_t value)
{
  float conv_value;

  conv_value=(value*VOLTAGE_DELTA-TEMP_V25)*TEMP_INV_SLOPE+25.0;

  return conv_value*(1<<10);// fixed point format 6 | 10
}

uint16_t adc_convert_vrefint(uint16_t value)
{
  float conv_value;

  conv_value=value*VOLTAGE_DELTA;

  return conv_value*(1<<12);
}

uint16_t adc_convert_voltage(uint16_t value)
{
  float conv_value;

  conv_value=value*VOLTAGE_DELTA*VOLTAGE_GAIN;

  return conv_value*(1<<12);
}

// interrupt handlers
void ADC_TIMER_IRQHandler(void)
{
  uint32_t capture;

  if(__HAL_TIM_GET_FLAG(&ADC_TIM_Handle, TIM_FLAG_CC4) != RESET)
  {
    if(__HAL_TIM_GET_IT_SOURCE(&ADC_TIM_Handle, TIM_IT_CC4) !=RESET)
    {
      __HAL_TIM_CLEAR_IT(&ADC_TIM_Handle, TIM_IT_CC4);
      capture = HAL_TIM_ReadCapturedValue(&ADC_TIM_Handle, TIM_CHANNEL_4);
      __HAL_TIM_SET_COMPARE(&ADC_TIM_Handle, TIM_CHANNEL_4, (capture + adc_period_ms));
    }
  }
  /* TIM Update event */
  if(__HAL_TIM_GET_FLAG(&ADC_TIM_Handle, TIM_FLAG_UPDATE) != RESET)
    if(__HAL_TIM_GET_IT_SOURCE(&ADC_TIM_Handle, TIM_IT_UPDATE) !=RESET)
      __HAL_TIM_CLEAR_IT(&ADC_TIM_Handle, TIM_IT_UPDATE);
  /* TIM Break input event */
  if(__HAL_TIM_GET_FLAG(&ADC_TIM_Handle, TIM_FLAG_BREAK) != RESET)
    if(__HAL_TIM_GET_IT_SOURCE(&ADC_TIM_Handle, TIM_IT_BREAK) !=RESET)
      __HAL_TIM_CLEAR_IT(&ADC_TIM_Handle, TIM_IT_BREAK);
  /* TIM Trigger detection event */
  if(__HAL_TIM_GET_FLAG(&ADC_TIM_Handle, TIM_FLAG_TRIGGER) != RESET)
    if(__HAL_TIM_GET_IT_SOURCE(&ADC_TIM_Handle, TIM_IT_TRIGGER) !=RESET)
      __HAL_TIM_CLEAR_IT(&ADC_TIM_Handle, TIM_IT_TRIGGER);
  /* TIM commutation event */
  if(__HAL_TIM_GET_FLAG(&ADC_TIM_Handle, TIM_FLAG_COM) != RESET)
    if(__HAL_TIM_GET_IT_SOURCE(&ADC_TIM_Handle, TIM_IT_COM) !=RESET)
      __HAL_TIM_CLEAR_IT(&ADC_TIM_Handle, TIM_FLAG_COM);
}

void ADC_DMA_IRQHandler(void)
{
  uint16_t value;

  /* Transfer Error Interrupt management ***************************************/
  if(__HAL_DMA_GET_FLAG(&hdma_adc1, __HAL_DMA_GET_TE_FLAG_INDEX(&hdma_adc1)) != RESET)
    if(__HAL_DMA_GET_IT_SOURCE(&hdma_adc1, DMA_IT_TE) != RESET)
      __HAL_DMA_CLEAR_FLAG(&hdma_adc1, __HAL_DMA_GET_TE_FLAG_INDEX(&hdma_adc1));
  /* FIFO Error Interrupt management ******************************************/
  if(__HAL_DMA_GET_FLAG(&hdma_adc1, __HAL_DMA_GET_FE_FLAG_INDEX(&hdma_adc1)) != RESET)
    if(__HAL_DMA_GET_IT_SOURCE(&hdma_adc1, DMA_IT_FE) != RESET)
      __HAL_DMA_CLEAR_FLAG(&hdma_adc1, __HAL_DMA_GET_FE_FLAG_INDEX(&hdma_adc1));
  /* Direct Mode Error Interrupt management ***********************************/
  if(__HAL_DMA_GET_FLAG(&hdma_adc1, __HAL_DMA_GET_DME_FLAG_INDEX(&hdma_adc1)) != RESET)
    if(__HAL_DMA_GET_IT_SOURCE(&hdma_adc1, DMA_IT_DME) != RESET)
      __HAL_DMA_CLEAR_FLAG(&hdma_adc1, __HAL_DMA_GET_DME_FLAG_INDEX(&hdma_adc1));
  /* Half Transfer Complete Interrupt management ******************************/
  if(__HAL_DMA_GET_FLAG(&hdma_adc1, __HAL_DMA_GET_HT_FLAG_INDEX(&hdma_adc1)) != RESET)
    if(__HAL_DMA_GET_IT_SOURCE(&hdma_adc1, DMA_IT_HT) != RESET)
      __HAL_DMA_CLEAR_FLAG(&hdma_adc1, __HAL_DMA_GET_HT_FLAG_INDEX(&hdma_adc1));
  /* Transfer Complete Interrupt management ***********************************/
  if(__HAL_DMA_GET_FLAG(&hdma_adc1, __HAL_DMA_GET_TC_FLAG_INDEX(&hdma_adc1)) != RESET)
    if(__HAL_DMA_GET_IT_SOURCE(&hdma_adc1, DMA_IT_TC) != RESET)
    {
      __HAL_DMA_CLEAR_FLAG(&hdma_adc1, __HAL_DMA_GET_TC_FLAG_INDEX(&hdma_adc1));
      value=adc_convert_voltage(adc_data[0]&0x0000FFFF);
      ram_data[BIOLOID_ADC_CH1_L]=value%256;
      ram_data[BIOLOID_ADC_CH1_H]=value/256;
      value=adc_convert_voltage((adc_data[0]&0xFFFF0000)>>16);
      ram_data[BIOLOID_ADC_CH2_L]=value%256;
      ram_data[BIOLOID_ADC_CH2_H]=value/256;
      value=adc_convert_voltage(adc_data[1]&0x0000FFFF);
      ram_data[BIOLOID_ADC_CH3_L]=value%256;
      ram_data[BIOLOID_ADC_CH3_H]=value/256;
      value=adc_convert_voltage((adc_data[1]&0xFFFF0000)>>16);
      ram_data[BIOLOID_ADC_CH4_L]=value%256;
      ram_data[BIOLOID_ADC_CH4_H]=value/256;
      value=adc_convert_temperature(adc_data[2]&0x0000FFFF);
      ram_data[BIOLOID_ADC_TEMP_L]=value%256;
      ram_data[BIOLOID_ADC_TEMP_H]=value/256;
      value=adc_convert_voltage((adc_data[2]&0xFFFF0000)>>16);
      ram_data[BIOLOID_ADC_CH6_L]=value%256;
      ram_data[BIOLOID_ADC_CH6_H]=value/256;
      value=adc_convert_vrefint(adc_data[3]&0x0000FFFF);
      ram_data[BIOLOID_ADC_VREF_L]=value%256;
      ram_data[BIOLOID_ADC_VREF_H]=value/256;
      value=adc_convert_voltage((adc_data[3]&0xFFFF0000)>>16);
      ram_data[BIOLOID_ADC_CH8_L]=value%256;
      ram_data[BIOLOID_ADC_CH8_H]=value/256;
    }
}

// public functions
void adc_init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct; 	
  ADC_MultiModeTypeDef multimode;
  ADC_ChannelConfTypeDef sConfig;
  TIM_ClockConfigTypeDef sClockSourceConfig;
  TIM_MasterConfigTypeDef sMasterConfig;
  TIM_OC_InitTypeDef TIM_OCInitStructure;
  uint32_t capture;

  /* initialize the internal variables */
  adc_period_ms=840;// equivalent to 10 ms
  ram_data[BIOLOID_ADC_PERIOD]=10;

  /* enable clocks */
  ADC1_CH1_ENABLE_PORT_CLK;
  ADC1_CH2_ENABLE_PORT_CLK;
  ADC2_CH1_ENABLE_PORT_CLK;
  ADC2_CH2_ENABLE_PORT_CLK;
  ADC2_CH3_ENABLE_PORT_CLK;
  ADC2_CH4_ENABLE_PORT_CLK;

  ADC_ENABLE_DMA_CLK;

  ADC1_ENABLE_CLK;
  /* configure ADC1 */
  hadc1.Instance = ADC1;
  hadc1.Init.ClockPrescaler = ADC_CLOCKPRESCALER_PCLK_DIV8;
  hadc1.Init.Resolution = ADC_RESOLUTION12b;
  hadc1.Init.ScanConvMode = ENABLE;
  hadc1.Init.ContinuousConvMode = DISABLE;// because it is triggered by a timer
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_RISINGFALLING;
  hadc1.Init.ExternalTrigConv = ADC_EXTERNALTRIGCONV_T4_CC4;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 4;
  hadc1.Init.DMAContinuousRequests = ENABLE;
  hadc1.Init.EOCSelection = EOC_SEQ_CONV;
  HAL_ADC_Init(&hadc1);

  multimode.Mode = ADC_DUALMODE_REGSIMULT;
  multimode.DMAAccessMode = ADC_DMAACCESSMODE_2;
  multimode.TwoSamplingDelay = ADC_TWOSAMPLINGDELAY_20CYCLES;
  HAL_ADCEx_MultiModeConfigChannel(&hadc1, &multimode);

  /* configure ADC1 channels */
  sConfig.Channel = ADC1_CH1;
  sConfig.Rank = 1;
  sConfig.SamplingTime = ADC_SAMPLETIME_480CYCLES;
  HAL_ADC_ConfigChannel(&hadc1, &sConfig);

  sConfig.Channel = ADC1_CH2;
  sConfig.Rank = 2;
  HAL_ADC_ConfigChannel(&hadc1, &sConfig);

  sConfig.Channel = ADC1_CH3;
  sConfig.Rank = 3;
  HAL_ADC_ConfigChannel(&hadc1, &sConfig);

  sConfig.Channel = ADC1_CH4;
  sConfig.Rank = 4;
  HAL_ADC_ConfigChannel(&hadc1, &sConfig);

  /* configure GPIO */
  GPIO_InitStruct.Pin = ADC1_CH1_PIN;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(ADC1_CH1_PORT, &GPIO_InitStruct);
  GPIO_InitStruct.Pin = ADC1_CH2_PIN;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(ADC1_CH2_PORT, &GPIO_InitStruct);

  ADC2_ENABLE_CLK;
  /* configure ADC1 */
  hadc2.Instance = ADC2;
  hadc2.Init.ClockPrescaler = ADC_CLOCKPRESCALER_PCLK_DIV8;
  hadc2.Init.Resolution = ADC_RESOLUTION12b;
  hadc2.Init.ScanConvMode = ENABLE;
  hadc2.Init.ContinuousConvMode = DISABLE;// because it is triggered by a timer
  hadc2.Init.DiscontinuousConvMode = DISABLE;
  hadc2.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc2.Init.NbrOfConversion = 4;
  hadc2.Init.DMAContinuousRequests = ENABLE;
  hadc2.Init.EOCSelection = EOC_SEQ_CONV;
  HAL_ADC_Init(&hadc2);

  multimode.Mode = ADC_DUALMODE_REGSIMULT;
  multimode.DMAAccessMode = ADC_DMAACCESSMODE_2;
  multimode.TwoSamplingDelay = ADC_TWOSAMPLINGDELAY_20CYCLES;
  HAL_ADCEx_MultiModeConfigChannel(&hadc2, &multimode);

  /* configure ADC2 channels */
  sConfig.Channel = ADC2_CH1;
  sConfig.Rank = 1;
  sConfig.SamplingTime = ADC_SAMPLETIME_480CYCLES;
  HAL_ADC_ConfigChannel(&hadc2, &sConfig);

  sConfig.Channel = ADC2_CH2;
  sConfig.Rank = 2;
  HAL_ADC_ConfigChannel(&hadc2, &sConfig);

  sConfig.Channel = ADC2_CH3;
  sConfig.Rank = 3;
  HAL_ADC_ConfigChannel(&hadc2, &sConfig);

  sConfig.Channel = ADC2_CH4;
  sConfig.Rank = 4;
  HAL_ADC_ConfigChannel(&hadc2, &sConfig);

  /* configure GPIO */
  GPIO_InitStruct.Pin = ADC2_CH1_PIN;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(ADC2_CH1_PORT, &GPIO_InitStruct);
  GPIO_InitStruct.Pin = ADC2_CH2_PIN;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(ADC2_CH2_PORT, &GPIO_InitStruct);
  GPIO_InitStruct.Pin = ADC2_CH3_PIN;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(ADC2_CH3_PORT, &GPIO_InitStruct);
  GPIO_InitStruct.Pin = ADC2_CH4_PIN;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(ADC2_CH4_PORT, &GPIO_InitStruct);

  /* initialize DMA */
  hdma_adc1.Instance = DMA2_Stream0;
  hdma_adc1.Init.Channel = DMA_CHANNEL_0;
  hdma_adc1.Init.Direction = DMA_PERIPH_TO_MEMORY;
  hdma_adc1.Init.PeriphInc = DMA_PINC_DISABLE;
  hdma_adc1.Init.MemInc = DMA_MINC_ENABLE;
  hdma_adc1.Init.PeriphDataAlignment = DMA_PDATAALIGN_WORD;
  hdma_adc1.Init.MemDataAlignment = DMA_MDATAALIGN_WORD;
  hdma_adc1.Init.Mode = DMA_CIRCULAR;
  hdma_adc1.Init.Priority = DMA_PRIORITY_LOW;
  hdma_adc1.Init.FIFOMode = DMA_FIFOMODE_DISABLE;
  HAL_DMA_Init(&hdma_adc1);

  __HAL_LINKDMA(&hadc1,DMA_Handle,hdma_adc1);

  HAL_NVIC_SetPriority(ADC_DMA_IRQn, 3, 1);
  HAL_NVIC_EnableIRQ(ADC_DMA_IRQn);

  /* configure the timer to generate the internal trigger */
  ENABLE_ADC_TIMER_CLK;
  ADC_TIM_Handle.Instance=ADC_TIMER;
  ADC_TIM_Handle.Init.Period = 0xFFFF;
  ADC_TIM_Handle.Init.Prescaler = 1000;
  ADC_TIM_Handle.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  ADC_TIM_Handle.Init.CounterMode = TIM_COUNTERMODE_UP;
  HAL_NVIC_SetPriority(ADC_TIMER_IRQn, 3, 1);
  HAL_NVIC_EnableIRQ(ADC_TIMER_IRQn);
  /* use the internal clock */
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  HAL_TIM_ConfigClockSource(&ADC_TIM_Handle, &sClockSourceConfig);
  HAL_TIM_OC_Init(&ADC_TIM_Handle);
  /* disable master/slave mode */
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_OC4REF;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  HAL_TIMEx_MasterConfigSynchronization(&ADC_TIM_Handle, &sMasterConfig);
  /* configure ouptut counter channel 4 */
  TIM_OCInitStructure.OCMode = TIM_OCMODE_TOGGLE;
  TIM_OCInitStructure.OCPolarity = TIM_OCPOLARITY_HIGH;
  TIM_OCInitStructure.OCFastMode = TIM_OCFAST_DISABLE;
  capture = HAL_TIM_ReadCapturedValue(&ADC_TIM_Handle, TIM_CHANNEL_4);
  TIM_OCInitStructure.Pulse = capture+adc_period_ms;
  HAL_TIM_OC_ConfigChannel(&ADC_TIM_Handle, &TIM_OCInitStructure,TIM_CHANNEL_4);

  /* start ADC operation */
  HAL_ADC_Start(&hadc2);
  HAL_ADCEx_MultiModeStart_DMA(&hadc1,adc_data,4);
  adc_start();
}

void adc_start(void)
{
  ram_data[BIOLOID_ADC_CNTRL]|=0x01;
  HAL_TIM_OC_Start_IT(&ADC_TIM_Handle, TIM_CHANNEL_4);
}

void adc_set_period(uint8_t period_ms)
{
  adc_period_ms=period_ms*84;
  ram_data[BIOLOID_ADC_PERIOD]=period_ms;
}

inline uint8_t adc_get_period(void)
{
  return ram_data[BIOLOID_ADC_PERIOD];
}

uint16_t adc_get_channel(adc_ch_t channel)
{
  uint16_t value;

  switch(channel)
  {
    case ADC_CH1: value=ram_data[BIOLOID_ADC_CH1_L]+ram_data[BIOLOID_ADC_CH1_H]*256;
                  break;
    case ADC_CH2: value=ram_data[BIOLOID_ADC_CH2_L]+ram_data[BIOLOID_ADC_CH2_H]*256;
                  break;
    case ADC_CH3: value=ram_data[BIOLOID_ADC_CH3_L]+ram_data[BIOLOID_ADC_CH3_H]*256;
                  break;
    case ADC_CH4: value=ram_data[BIOLOID_ADC_CH4_L]+ram_data[BIOLOID_ADC_CH4_H]*256;
                  break;
    case ADC_CH6: value=ram_data[BIOLOID_ADC_CH6_L]+ram_data[BIOLOID_ADC_CH6_H]*256;
                  break;
    case ADC_CH8: value=ram_data[BIOLOID_ADC_CH8_L]+ram_data[BIOLOID_ADC_CH8_H]*256;
                  break;
    default: value=0x0000;
  }

  return value;
}

uint16_t adc_get_channel_raw(adc_ch_t channel)
{
  uint16_t value;

  switch(channel)
  {
    case ADC_CH1: value=adc_data[0]&0x0000FFFF;
                  break;
    case ADC_CH2: value=(adc_data[0]&0xFFFF0000)>>16;
                  break;
    case ADC_CH3: value=adc_data[1]&0x0000FFFF;
                  break;
    case ADC_CH4: value=(adc_data[1]&0xFFFF0000)>>16;
                  break;
    case ADC_CH6: value=(adc_data[2]&0xFFFF0000)>>16;
                  break;
    case ADC_CH8: value=(adc_data[3]&0xFFFF0000)>>16;
                  break;
    default: value=0x0000;
  }
  
  return value;
}

uint16_t adc_get_temperature(void)
{
  return ram_data[BIOLOID_ADC_TEMP_L]+ram_data[BIOLOID_ADC_TEMP_H]*256;
}

void adc_stop(void)
{
  ram_data[BIOLOID_ADC_CNTRL]&=0xFE;
  HAL_TIM_OC_Stop_IT(&ADC_TIM_Handle, TIM_CHANNEL_4);
}

// operation functions
uint8_t adc_in_range(unsigned short int address,unsigned short int length)
{
  return ram_in_window(ADC_BASE_ADDRESS,ADC_MEM_LENGTH,address,length);
}

void adc_process_write_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{
  if(ram_in_range(BIOLOID_ADC_CNTRL,address,length))
  {
    if(data[BIOLOID_ADC_CNTRL-address]&ADC_ENABLE)
      adc_start();
    else
      adc_stop();
  }
  if(ram_in_range(BIOLOID_ADC_PERIOD,address,length))
    adc_set_period(data[BIOLOID_ADC_PERIOD-address]);
}
