#include "bioloid_gyro.h"
#include "ram.h"

#define GYRO_MAX_CAL_SAMPLES 256

/* private variables */
uint32_t gyro_fb_offset;
uint32_t gyro_lr_offset;

/* private functions */

/* public functions */
void gyro_init(void)
{
  /* initialize internal variables */
  gyro_fb_offset=0x0000;
  gyro_lr_offset=0x0000;
  /* calibrate the sensor */
  gyro_calibrate();
}

inline adc_ch_t gyro_get_fb_adc_channel(void)
{
  return ram_data[BIOLIOD_GYRO_FB_ADC_CH];
}

inline adc_ch_t gyro_get_lr_adc_channel(void)
{
  return ram_data[BIOLIOD_GYRO_LR_ADC_CH];
}

void gyro_calibrate(void)
{
  adc_ch_t fb_ch=ram_data[BIOLIOD_GYRO_FB_ADC_CH],lr_ch=ram_data[BIOLIOD_GYRO_LR_ADC_CH];
  uint16_t i;

  gyro_fb_offset=0x0000;
  gyro_lr_offset=0x0000;
  // dummy delay to allow enough time to the ADC to get new data
  HAL_Delay(2*adc_get_period());    
  for(i=0;i<GYRO_MAX_CAL_SAMPLES;i++)
  {
    gyro_fb_offset+=adc_get_channel_raw(fb_ch);
    gyro_lr_offset+=adc_get_channel_raw(lr_ch);
    /* wait for the next sample */
    HAL_Delay(adc_get_period());    
  }
  gyro_fb_offset/=GYRO_MAX_CAL_SAMPLES;
  gyro_lr_offset/=GYRO_MAX_CAL_SAMPLES;
}

inline void gyro_get_offsets(uint16_t *fb_offset,uint16_t *lr_offset)
{
  *fb_offset=gyro_fb_offset;
  *lr_offset=gyro_lr_offset;
}

// operation functions
uint8_t gyro_in_range(unsigned short int address, unsigned short int length)
{
  if(ram_in_window(GYRO_BASE_ADDRESS,GYRO_MEM_LENGTH,address,length) ||
     ram_in_window(BIOLIOD_GYRO_FB_ADC_CH,BIOLIOD_GYRO_LR_ADC_CH,address,length))
    return 0x01;
  else
    return 0x00;
}

void gyro_process_read_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{

}

void gyro_process_write_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{

}

